//////////////////////////////////////////////////////////////////////////////
// GpgaEngine.h	- GPGA engine class
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// slave thread data structure

typedef struct {
	// high-performace timer frequency (obtained from the master)
	LARGE_INTEGER	llCounterFreq;

	// total running time (cumulative, for all iterations, in msec)
	LARGE_INTEGER	llTotalThreadRunTime;

	// pointer to the slave operator
	CSlaveOperatorBase*	pSlaveOperator;

	// thread numbers
	DWORD			dwThreadNo;
	DWORD			dwTotalThreads;

	// event indicating that a thread has finished with slave operator
	HANDLE			hThreadFinished;
	// event used to signal to the thread to start evaluating
	HANDLE			hStartEvent;
	// semaphore used to signal the threads to exit
	HANDLE			hExitSemaphore;

	// if this flag is set, a slave thread is signaling to the master that 
	// it should stop with the iterations
	BOOL			bStopMaster;

} THREAD_DATA, *PTHREAD_DATA;

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// CGpgaEngine	- GPGA engine class
// The class incorporates GPGA functionality.

class CGpgaEngine {
	public:
		// construction/destruction
		CGpgaEngine(COperatorBase& opInitialize, COperatorBase& opMaster, CSlaveOperatorBase& opSlave, DWORD dwSlaveThreads);

		~CGpgaEngine();

		// runs the engine for specified number of iterations (max)
		DWORD Run(DWORD dwIterations);

		// returns the statistics for the previous run
		DWORD* GetStatistics(LARGE_INTEGER* pllMasterTime);

	private:

		// slave thread(s) function
		static DWORD WINAPI SlaveThread(LPVOID lpThreadData);

		COperatorBase&		m_opInitialize;
		COperatorBase&		m_opMaster;
		CSlaveOperatorBase& m_opSlave;

		// total number of slave threads
		DWORD			m_dwSlaveThreads;
		// thread data (one for each :-)
		THREAD_DATA*	m_pThreadsData;
		// thread handles
		HANDLE*			m_phThreads;
		// handles used by the threads to indicate that they have finished
		// with running the slave operator
		HANDLE*			m_phThreadsFinished;

		// events used to signal to the slave threads to run the
		// slave operator
		HANDLE*			m_phThreadsStart;
		// semaphore used to indicate to the slave threads that they 
		// should exit
		HANDLE			m_hThreadsExitSemaphore;

		// statistics stuff...
		LARGE_INTEGER	m_llCounterFreq;
		LARGE_INTEGER	m_llTotalMasterRunTime;	
		LARGE_INTEGER	m_llStartTime;
		LARGE_INTEGER	m_llEndTime;
};

//////////////////////////////////////////////////////////////////////////////

