// main.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "GpgaIndividual.h"
#include "GpgaPopulation.h"
#include "GpgaDefaultOperators.h"
#include "GpgaEngine.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MASTER_ITER	200
#define	SLAVE_ITER	100
#define THREADS		2
#define	DIMEN		40

int main(int argc, char* argv[])
{
	// turn on memory allocation tracking
#	ifdef _DEBUG
		int nDbgFlag = ::_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
		nDbgFlag |= _CRTDBG_LEAK_CHECK_DF;
		::_CrtSetDbgFlag(nDbgFlag);
#	endif

		double pLowerLimits[] = {-10, -10, -10, -10, -10, 
								-10, -10, -10, -10, -10, 
								-10, -10, -10, -10, -10, 
								-10, -10, -10, -10, -10, 
								-10, -10, -10, -10, -10, 
								-10, -10, -10, -10, -10, 
								-10, -10, -10, -10, -10, 
								-10, -10, -10, -10, -10};
		double pUpperLimits[] = {10, 10, 10, 10, 10, 
								10, 10, 10, 10, 10, 
								10, 10, 10, 10, 10, 
								10, 10, 10, 10, 10, 
								10, 10, 10, 10, 10, 
								10, 10, 10, 10, 10, 
								10, 10, 10, 10, 10, 
								10, 10, 10, 10, 10};
	double dBitMutationRate = 0.03;

	CPopulation<DWORD, double>	thePopulation(50, DIMEN);

	CDefaultEvalOperator		opEval(
									thePopulation, 
									pLowerLimits, 
									pUpperLimits, 
									40004, 
									1);
	CIndividual<DWORD, double>* pInd;
	CDefaultInitOperator theInitOperator(thePopulation, opEval);
	CDefaultSelectionOperator opSelection(
								thePopulation, 
								opEval);
	CDefaultSlaveOperator	opSlave(opSelection, SLAVE_ITER);
	CDefaultMutationOperator opMutation(
								thePopulation, 
								opEval, 
								dBitMutationRate);

	DWORD dwDim;
	double* pdPoint;
	double dDelta;
	DWORD* pThreadTimes;
	LARGE_INTEGER				llMasterTime;
	DWORD i;


	pInd = thePopulation.GetBestIndividualPtr();

	dwDim = pInd->GetChromosomeDim();
	pdPoint = new double[dwDim];

	CGpgaEngine	theEngine(theInitOperator, opMutation, opSlave, THREADS);

	theEngine.Run(MASTER_ITER);
	pThreadTimes = theEngine.GetStatistics(&llMasterTime);
	for (i = 0; i < THREADS; i++) {
		printf("Thread No. %3i total time: %5i msec\n", i, pThreadTimes[i]);
		printf("Thread No. %3i avg   time: %5i msec\n", i, pThreadTimes[i] / MASTER_ITER);
	}
	delete pThreadTimes;
	
	pInd = thePopulation.GetBestIndividualPtr();
	for (i = 0; i < dwDim; i++) {
		dDelta = pUpperLimits[i] - pLowerLimits[i];
		pdPoint[i] = dDelta / 0xFFFFFFFF * (pInd->GetChromosomePtr())[i] + pLowerLimits[i];
	}
	printf("Best individual fitness: %15.10f\n", pInd->GetFitness());


	theEngine.Run(MASTER_ITER);
	pThreadTimes = theEngine.GetStatistics(&llMasterTime);
	for (i = 0; i < THREADS; i++) {
		printf("Thread No. %3i total time: %5i msec\n", i, pThreadTimes[i]);
		printf("Thread No. %3i avg   time: %5i msec\n", i, pThreadTimes[i] / MASTER_ITER);
	}
	delete pThreadTimes;
							
	pInd = thePopulation.GetBestIndividualPtr();
	for (i = 0; i < dwDim; i++) {
		dDelta = pUpperLimits[i] - pLowerLimits[i];
		pdPoint[i] = dDelta / 0xFFFFFFFF * (pInd->GetChromosomePtr())[i] + pLowerLimits[i];
	}
	printf("Best individual fitness: %15.10f\n", pInd->GetFitness());

	delete pdPoint;

	return 0;
}
