//////////////////////////////////////////////////////////////////////////////
// GpgaNQueen.cpp : Defines the entry point for the console application.
//
// This program uses GPGA engine to solve n-Queen problem (place N queens on
// an NxN chessboard in a such way that no queens attack each other)

#include "stdafx.h"
#include <conio.h>
#include "GpgaIndividual.h"
#include "GpgaPopulation.h"
#include "GpgaNQueenOperators.h"
#include "GpgaEngine.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//#define POP_SIZE	1000
//#define	QUEENS		30
//#define MUT_PROB	0.05
//#define	SLAVE_ITER	100
//#define SLAVES		3

//////////////////////////////////////////////////////////////////////////////
#define IS_SPACE(c)	((c) == ' ' || (c) == '\t' || (c) == '\n' || (c) == '\r')
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// keywords

#define KEYWORDS	5
CHAR* Keywords[] = {
	"queens", 
	"population_size", 
	"slave_iterations", 
	"slave_threads", 
	"mutation_rate"
};

//////////////////////////////////////////////////////////////////////////////

BOOL g_bStat = FALSE;


BOOL WINAPI MyHandlerRoutine(DWORD dwCtrlType)
{
	if (dwCtrlType == CTRL_C_EVENT) {
		g_bStat = TRUE;
		return TRUE;
	} else {
		return FALSE;
	}
}

//////////////////////////////////////////////////////////////////////////////

int main(int argc, char* argv[])
{
	// turn on memory allocation tracking
#	ifdef _DEBUG
		int nDbgFlag = ::_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
		nDbgFlag |= _CRTDBG_LEAK_CHECK_DF;
		::_CrtSetDbgFlag(nDbgFlag);
#	endif

	SetConsoleCtrlHandler(MyHandlerRoutine,TRUE);


	// we need two cmd-line parameters, input and output files
	if (argc != 3) {
		printf("Usage: GpgaNQueen.exe <input file> <output file>\n For details see sample input file.\n");
		return -1;
	}

	//////////////////////////////////////////////////////////////////////////
	// GA variables

	DWORD	dwQueens;
	DWORD	dwPopulationSize;
	DWORD	dwSlaveIterations;
	DWORD	dwSlaveThreads;
	double	dMutationRate;

	//////////////////////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////////////////////
	// Read config file and set variables

	// read the input file
	HANDLE	hInFile;
	CHAR*	pszFileData;
	DWORD	dwFileSize;

	if ((hInFile = ::CreateFile(
					argv[1], 
					GENERIC_READ, 
					FILE_SHARE_READ, 
					NULL, 
					OPEN_EXISTING, 
					FILE_FLAG_SEQUENTIAL_SCAN, 
					NULL)) == NULL) {

		printf("Unable to open input file!\n");
		return -1;
	}

	dwFileSize = ::GetFileSize(hInFile, NULL);
	pszFileData = new CHAR[dwFileSize + 1];
	::ZeroMemory(pszFileData, dwFileSize + 1);

	::ReadFile(hInFile, pszFileData, dwFileSize, &dwFileSize, NULL);

	// let's parse the data now
	CHAR*	pTmp = pszFileData;
	CHAR	szToken[256];
	BOOL	bSkipToEOL = FALSE;

	DWORD	i;

	while (pTmp[0] != '\0') {

		if (bSkipToEOL) {
			// skip the comments
			if (pTmp[0] == '\n') bSkipToEOL = FALSE;
			++pTmp;
		} else {
			if (pTmp[0] == '#') {
				// comment
				bSkipToEOL = TRUE;
				++pTmp;
			} else if (IS_SPACE(pTmp[0])) {
				++pTmp;
			} else {
				// we have some text...
				sscanf(pTmp, "%s", szToken);

				// ... is it a keyword?
				for (i = 0; i < KEYWORDS; ++i) {
					if (!_strcmpi(szToken, Keywords[i])) break;
				}
				pTmp += strlen(szToken);

				// let's see
				switch (i) {
					// queens
					case 0	:
						sscanf(pTmp, "%i", &dwQueens);
						bSkipToEOL = TRUE;
						break;

					// population_size
					case 1	:
						sscanf(pTmp, "%i", &dwPopulationSize);
						bSkipToEOL = TRUE;
						break;

					// slave_iterations
					case 2	:
						sscanf(pTmp, "%i", &dwSlaveIterations);
						bSkipToEOL = TRUE;
						break;

					// slave_threads
					case 3	:
						sscanf(pTmp, "%i", &dwSlaveThreads);
						bSkipToEOL = TRUE;
						break;

					// mutation_rate
					case 4	:
						sscanf(pTmp, "%lf", &dMutationRate);
						bSkipToEOL = TRUE;
						break;

					default : bSkipToEOL = TRUE;
				};
			}
		}
	}

	// cleanup file stuff
	delete pszFileData;
	::CloseHandle(hInFile);

	//////////////////////////////////////////////////////////////////////////



	//////////////////////////////////////////////////////////////////////////
	// Create population, operators and engine

	CPopulation<WORD, float>	Population(dwPopulationSize, dwQueens);

	CNQeenEvalOperator			EvalOperator(Population);

	CNQueenInitOperator			InitOperator(Population, EvalOperator);

	CNQueenSelectionOperator	SelectionOperator(
									Population, 
									EvalOperator);

	CNQueenSlaveOperator		SlaveOperator(
									SelectionOperator, 
									dwSlaveIterations);

	CNQueenMutationOperator		MutationOperator(
									Population, 
									EvalOperator, 
									dMutationRate);

	CGpgaEngine					GpgaEngine(
									InitOperator, 
									MutationOperator, 
									SlaveOperator, 
									dwSlaveThreads);

	//////////////////////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////////////////////
	// Run the engine, and create output file with results

	FILE*	hOutputFile;

	CIndividual<WORD, float>* pBestIndividual;
	WORD*	pChrom;

	DWORD*			pThreadsTimes;
	LARGE_INTEGER	llMasterTime;
	DWORD			dwTotalIters = 0;

	do {
		dwTotalIters += GpgaEngine.Run(1000);
		pBestIndividual = Population.GetBestIndividualPtr();
		printf("Iter: %i, best ind fitness: %f\n", dwTotalIters, pBestIndividual->GetFitness());
		pChrom = pBestIndividual->GetChromosomePtr();

		if (g_bStat) {
			FILE*	hFile;
			CHAR	szFileName[256];
			CIndividual<WORD, float>* pInd;
			float fTotalFitness = 0.0;

			sprintf(szFileName, "F:\\Smece\\%i.txt", dwTotalIters);
			hFile = fopen(szFileName, "wt");

			for (i = 0; i < dwPopulationSize; ++i) {
				pInd = Population.GetIndividualPtr(i);

				fTotalFitness += pInd->GetFitness();
				for (DWORD j = 0; j < dwQueens; ++j) {
					fprintf(hFile, "%4i ", pInd->GetChromosomePtr()[j]);
				}
				fprintf(hFile, "%f\n", pInd->GetFitness());
			}

			fprintf(hFile, "Avg. fitenss: %f\n", fTotalFitness/dwPopulationSize);

			fclose(hFile);
			g_bStat = FALSE;
		}

	} while (pBestIndividual->GetFitness() != 0);

	pThreadsTimes			= GpgaEngine.GetStatistics(&llMasterTime);

	hOutputFile = fopen(argv[2], "wt");

	fprintf(hOutputFile, "##############################################################################\n");
	fprintf(hOutputFile, "# Number of queens:         %i\n", dwQueens);
	fprintf(hOutputFile, "##############################################################################\n");
	fprintf(hOutputFile, "# Population size:          %i\n", dwPopulationSize);
	fprintf(hOutputFile, "# Mutation probability:     %.4lf\n", dMutationRate);
	fprintf(hOutputFile, "# Number of slave iter.:    %i\n", dwSlaveIterations);
	fprintf(hOutputFile, "# Number of slave threads:  %i\n", dwSlaveThreads);
	fprintf(hOutputFile, "# \n");
	fprintf(hOutputFile, "# Total master iterations:  %i\n", dwTotalIters);
	fprintf(hOutputFile, "##############################################################################\n");

	fprintf(hOutputFile, "\n\n");
	
	fprintf(hOutputFile, "##############################################################################\n");
	fprintf(hOutputFile, "# Master time: %I64i msec\n", llMasterTime.QuadPart);
	fprintf(hOutputFile, "##############################################################################\n");
	fprintf(hOutputFile, "# Threads' times\n");
	fprintf(hOutputFile, "#\n");

	for (i = 0; i < dwSlaveThreads; ++i) {
		fprintf(hOutputFile, "# Thread %i\n", i);
		fprintf(hOutputFile, "# Total running time: %i (msec)\n", pThreadsTimes[i]);
		fprintf(hOutputFile, "# Avg. running time: %i (msec/master iter.)\n", pThreadsTimes[i] / dwTotalIters);
		fprintf(hOutputFile, (i == dwSlaveThreads - 1 ? "" : "#\n"));
	}
	fprintf(hOutputFile, "##############################################################################\n");

	fprintf(hOutputFile, "\n\n");

	fprintf(hOutputFile, "##############################################################################\n");
	fprintf(hOutputFile, "# Solution:\n");
	fprintf(hOutputFile, "#\n# ");

	for (i = 0; i < dwQueens; ++i) {
		fprintf(hOutputFile, "%3i ", pChrom[i]);
	}

	fprintf(hOutputFile, "\n#\n");
	fprintf(hOutputFile, "##############################################################################\n");

	fclose(hOutputFile);
	delete pThreadsTimes;

	//////////////////////////////////////////////////////////////////////////

	printf("\nSolution:\n");
	for (i = 0; i < dwQueens; ++i) {
		printf("%3i ", pChrom[i]);
	}

	getch();

	return 0;
}
