//////////////////////////////////////////////////////////////////////////////
// GpgaNQueenOperators.h - operator classes for n-Queen problem
//
// This program uses GPGA engine to solve n-Queen problem (place N queens on
// an NxN chessboard in a such way that no queens attack each other)


#include "GpgaOperatorBase.h"


//////////////////////////////////////////////////////////////////////////////
// Initialization operator class

class CNQueenInitOperator : public COperatorBase {
	public:
		CNQueenInitOperator(CPopulation<WORD, float>& Population, COperatorBase& EvalOperator);
		~CNQueenInitOperator();

		DWORD Run();

	private:
		/////////////////////////////
		// operations
		void CreateRandomIndividuals();

		
		/////////////////////////////
		// data members
		CPopulation<WORD, float>&	m_Population;
		COperatorBase&				m_EvalOperator;
};

//////////////////////////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Slave operator class

class CNQueenSlaveOperator : public CSlaveOperatorBase {
	public:
		CNQueenSlaveOperator(COperatorBase& Selection, DWORD dwSlaveIterations);
		~CNQueenSlaveOperator();

		DWORD Run();
		DWORD Run(DWORD dwThreadNo, DWORD dwTotalThreads);

	private:
		/////////////////////////////
		// data members

		COperatorBase&	m_Selection;
		DWORD			m_dwSlaveIterations;
};

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Evaluation operator class
//
// This operator counts the number of conflicting queens on the board

class CNQeenEvalOperator : public CEvalOperatorBase {
	public:
		CNQeenEvalOperator(CPopulation<WORD, float>& Population);
		~CNQeenEvalOperator();

		DWORD Run();
		DWORD Run(CIndividual<class ChromClass, class FitClass>* pIndividual);

	private:
		/////////////////////////////
		// operations

		
		/////////////////////////////
		// data members

		// population pointer
		CPopulation<WORD, float>& m_Population;
};

//////////////////////////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Selection operator class
//
// Performs a 3-way tournamenet selection

class CNQueenSelectionOperator : public COperatorBase {
	public:
		CNQueenSelectionOperator(CPopulation<WORD, float>& Population, CEvalOperatorBase& EvalOperator);
		~CNQueenSelectionOperator();

		DWORD Run();

	private:

		/////////////////////////////
		// data members

		CPopulation<WORD, float>&	m_Population;
		CEvalOperatorBase&			m_EvalOperator;
};

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Crossover operator class
//
// Performs a custom n-Queen crossover

class CNQueenCrossoverOperator : public COperatorBase {
	public:
		CNQueenCrossoverOperator(CIndividual<WORD, float>* pParent1, CIndividual<WORD, float>* pParent2, CIndividual<WORD, float>* pOffspring);
		~CNQueenCrossoverOperator();

		DWORD Run();

	private:

		/////////////////////////////
		// data members

		CIndividual<WORD, float>* m_pParent1;
		CIndividual<WORD, float>* m_pParent2;
		CIndividual<WORD, float>* m_pOffspring;
};

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Mutation operator class
//
// Performs a simple n-Queen mutation on a population

class CNQueenMutationOperator : public COperatorBase {
	public:
		CNQueenMutationOperator(CPopulation<WORD, float>& Population, CEvalOperatorBase& EvalOperator, double dChromosomeMutationRate);
		~CNQueenMutationOperator();

		DWORD Run();

	private:

		/////////////////////////////
		// data members

		CPopulation<WORD, float>&	m_Population;
		CEvalOperatorBase&			m_EvalOperator;

		double	m_dChromosomeMutationRate;
};

//////////////////////////////////////////////////////////////////////////////
