//////////////////////////////////////////////////////////////////////////////
// GpgaIndividual.h	- template class for individual representation
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// CIndividual	- template class used for individual representation
//

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// class declaration

template <class ChromClass, class FitClass> 
class CIndividual {
	public:
		// construction/destruction
		CIndividual();
		virtual ~CIndividual();

		// allocates chromosome
		virtual void AllocChromosome(DWORD dwChromDim);

		// chromosome handling functions
		virtual ChromClass*	GetChromosomePtr();
		virtual DWORD		GetChromosomeDim();

		// fitness handling functions
		virtual const FitClass&	GetFitness();
		virtual void			SetFitness(const FitClass& Fitness);

	private:
		// chromosome data
		ChromClass*			m_pChromosome;
		DWORD				m_dwChromosomeDimension;

		// individual fitness
		FitClass			m_Fitness;
};

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// class definition (this *should* go to a .cpp file, but @#@!$ VC has a bug
// (see MSDN article Q239436)

//////////////////////////////////////////////////////////////////////////////
// default constructor

template <class ChromClass, class FitClass>
CIndividual<ChromClass, FitClass>::CIndividual()
: m_pChromosome(0)
{
}
	
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// destructor

template <class ChromClass, class FitClass>
CIndividual<ChromClass, FitClass>::~CIndividual()
{
	delete [] m_pChromosome;
}

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// AllocChromosome	- allocates chromosome
//
// DWORD dwChromosomeDimension	- chromosome vector dimension

template <class ChromClass, class FitClass>
void CIndividual<ChromClass, FitClass>::AllocChromosome(DWORD dwChromosomeDimension)
{
	// create chromosome
	m_dwChromosomeDimension = dwChromosomeDimension;
	m_pChromosome	= new ChromClass[m_dwChromosomeDimension];
}

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// GetChromosomePtr	- returns pointer to chromosome data

template <class ChromClass, class FitClass>
ChromClass* CIndividual<ChromClass, FitClass>::GetChromosomePtr()
{
	return m_pChromosome;
}

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// GetChromosomeDim	- returns chromosome vector dimension
//
// returns DWORD - chromosome vector dimension

template <class ChromClass, class FitClass>
DWORD CIndividual<ChromClass, FitClass>::GetChromosomeDim()
{
	return m_dwChromosomeDimension;
}

//////////////////////////////////////////////////////////////////////////////
// GetFitness	- returns reference to fitness data

template <class ChromClass, class FitClass>
const FitClass& CIndividual<ChromClass, FitClass>::GetFitness()
{
	return m_Fitness;
}

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// SetFitness	- sets fitness data
//
// const FitClass& Fitness	- fitness data

template <class ChromClass, class FitClass>
void CIndividual<ChromClass, FitClass>::SetFitness(const FitClass& Fitness)
{
	m_Fitness = Fitness;
}

//////////////////////////////////////////////////////////////////////////////
