//////////////////////////////////////////////////////////////////////////////
// GpgaRealFnOperators.h	- operator classes for real fn implementation
//                            (32-bit binary chromosome representation with
//                            real-number (double) fitness
//////////////////////////////////////////////////////////////////////////////

#include "GpgaOperatorBase.h"


//////////////////////////////////////////////////////////////////////////////
// Initialization operator class

class CRealFnInitOperator : public COperatorBase {
	public:
		CRealFnInitOperator(CPopulation<DWORD, double>& Population, COperatorBase& EvalOperator);
		~CRealFnInitOperator();

		DWORD Run();

	private:
		/////////////////////////////
		// operations
		void CreateRandomIndividuals();

		
		/////////////////////////////
		// data members
		CPopulation<DWORD, double>& m_Population;
		COperatorBase&				m_EvalOperator;
};

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Slave operator class

class CRealFnSlaveOperator : public CSlaveOperatorBase {
	public:
		CRealFnSlaveOperator(COperatorBase& Selection, DWORD dwSlaveIterations);
		~CRealFnSlaveOperator();

		DWORD Run();
		DWORD Run(DWORD dwThreadNo, DWORD dwTotalThreads);

	private:
		/////////////////////////////
		// data members

		COperatorBase&	m_Selection;
		DWORD			m_dwSlaveIterations;
};

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Evaluation operator class
//
// This operator class contains several test functions (see Evaluate function)

class CRealFnEvalOperator : public CEvalOperatorBase {
	public:
		CRealFnEvalOperator(CPopulation<DWORD, double>& Population, double* pdLowerLimits, double* pdUpperLimits, int nEvalFnID, int nFitnessFactor);
		~CRealFnEvalOperator();

		DWORD Run();
		DWORD Run(CIndividual<class ChromClass, class FitClass>* pIndividual);

		// clients may also use this function to decode individual data
		double* Decode(CIndividual<DWORD, double>* pIndividual);

	private:
		/////////////////////////////
		// operations

		double Evaluate(double* pdPoint);

		
		/////////////////////////////
		// data members

		// population pointer
		CPopulation<DWORD, double>& m_Population;

		// lower and upper limits used for individual decoding 
		double* m_pdLowerLimits;
		double* m_pdUpperLimits;

		// function ID that will be used for evaluation
		int		m_nEvalFnID;

		// 1 for maximization, -1 for minimization
		int		m_nFitnessFactor;
};

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Selection operator class
//
// Performs a 3-way tournamenet selection

class CRealFnSelectionOperator : public COperatorBase {
	public:
		CRealFnSelectionOperator(CPopulation<DWORD, double>& Population, CEvalOperatorBase& EvalOperator);
		~CRealFnSelectionOperator();

		DWORD Run();

	private:

		/////////////////////////////
		// data members

		CPopulation<DWORD, double>&	m_Population;
		CEvalOperatorBase&			m_EvalOperator;
};

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Crossover operator class
//
// Performs a uniform crossover

class CRealFnCrossoverOperator : public COperatorBase {
	public:
		CRealFnCrossoverOperator(CIndividual<DWORD, double>* pParent1, CIndividual<DWORD, double>* pParent2, CIndividual<DWORD, double>* pOffspring);
		~CRealFnCrossoverOperator();

		DWORD Run();

	private:

		/////////////////////////////
		// data members

		CIndividual<DWORD, double>* m_pParent1;
		CIndividual<DWORD, double>* m_pParent2;
		CIndividual<DWORD, double>* m_pOffspring;
};

//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Mutation operator class
//
// Performs a simple mutation on a population

class CRealFnMutationOperator : public COperatorBase {
	public:
		CRealFnMutationOperator(CPopulation<DWORD, double>& Population, CEvalOperatorBase& EvalOperator, double dBitMutationRate);
		~CRealFnMutationOperator();

		DWORD Run();

	private:

		/////////////////////////////
		// data members

		CPopulation<DWORD, double>&	m_Population;
		CEvalOperatorBase&			m_EvalOperator;

		double	m_dBitMutationRate;
};

//////////////////////////////////////////////////////////////////////////////
